library custom_theme;

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class CustomTheme<T> extends ThemeData {
  CustomTheme({
    @required ThemeData themeData,
    T userData,
  })  : _userData = userData,
        super.raw(
          brightness: themeData.brightness,
          primaryColor: themeData.primaryColor,
          primaryColorBrightness: themeData.primaryColorBrightness,
          primaryColorLight: themeData.primaryColorLight,
          primaryColorDark: themeData.primaryColorDark,
          canvasColor: themeData.canvasColor,
          accentColor: themeData.accentColor,
          accentColorBrightness: themeData.accentColorBrightness,
          scaffoldBackgroundColor: themeData.scaffoldBackgroundColor,
          bottomAppBarColor: themeData.bottomAppBarColor,
          cardColor: themeData.cardColor,
          dividerColor: themeData.dividerColor,
          focusColor: themeData.focusColor,
          hoverColor: themeData.hoverColor,
          highlightColor: themeData.highlightColor,
          splashColor: themeData.splashColor,
          splashFactory: themeData.splashFactory,
          selectedRowColor: themeData.selectedRowColor,
          unselectedWidgetColor: themeData.unselectedWidgetColor,
          disabledColor: themeData.disabledColor,
          buttonTheme: themeData.buttonTheme,
          buttonColor: themeData.buttonColor,
          toggleButtonsTheme: themeData.toggleButtonsTheme,
          secondaryHeaderColor: themeData.secondaryHeaderColor,
          textSelectionColor: themeData.textSelectionColor,
          cursorColor: themeData.cursorColor,
          textSelectionHandleColor: themeData.textSelectionHandleColor,
          backgroundColor: themeData.backgroundColor,
          dialogBackgroundColor: themeData.dialogBackgroundColor,
          indicatorColor: themeData.indicatorColor,
          hintColor: themeData.hintColor,
          errorColor: themeData.errorColor,
          toggleableActiveColor: themeData.toggleableActiveColor,
          textTheme: themeData.textTheme,
          primaryTextTheme: themeData.primaryTextTheme,
          accentTextTheme: themeData.accentTextTheme,
          inputDecorationTheme: themeData.inputDecorationTheme,
          iconTheme: themeData.iconTheme,
          primaryIconTheme: themeData.primaryIconTheme,
          accentIconTheme: themeData.accentIconTheme,
          sliderTheme: themeData.sliderTheme,
          tabBarTheme: themeData.tabBarTheme,
          tooltipTheme: themeData.tooltipTheme,
          cardTheme: themeData.cardTheme,
          chipTheme: themeData.chipTheme,
          platform: themeData.platform,
          materialTapTargetSize: themeData.materialTapTargetSize,
          applyElevationOverlayColor: themeData.applyElevationOverlayColor,
          pageTransitionsTheme: themeData.pageTransitionsTheme,
          appBarTheme: themeData.appBarTheme,
          bottomAppBarTheme: themeData.bottomAppBarTheme,
          colorScheme: themeData.colorScheme,
          dialogTheme: themeData.dialogTheme,
          floatingActionButtonTheme: themeData.floatingActionButtonTheme,
          typography: themeData.typography,
          cupertinoOverrideTheme: themeData.cupertinoOverrideTheme,
          snackBarTheme: themeData.snackBarTheme,
          bottomSheetTheme: themeData.bottomSheetTheme,
          popupMenuTheme: themeData.popupMenuTheme,
          bannerTheme: themeData.bannerTheme,
          dividerTheme: themeData.dividerTheme,
        );

  final T _userData;

  static CustomTheme lastSuccessfulTheme;

  static CustomTheme<T> of<T>(BuildContext context) {
    try {
      final theme = Theme.of(context) as CustomTheme<T>;
      lastSuccessfulTheme = theme;
      return theme;
    } on CastError catch (_) {
      return lastSuccessfulTheme;
    }
  }

  static T get<T>(BuildContext context) => CustomTheme.of<T>(context)._userData;

  @override
  CustomTheme<T> copyWith({
    Brightness brightness,
    Color primaryColor,
    Brightness primaryColorBrightness,
    Color primaryColorLight,
    Color primaryColorDark,
    Color accentColor,
    Brightness accentColorBrightness,
    Color canvasColor,
    Color scaffoldBackgroundColor,
    Color bottomAppBarColor,
    Color cardColor,
    Color dividerColor,
    Color focusColor,
    Color hoverColor,
    Color highlightColor,
    Color splashColor,
    InteractiveInkFeatureFactory splashFactory,
    Color selectedRowColor,
    Color unselectedWidgetColor,
    Color disabledColor,
    ButtonThemeData buttonTheme,
    ToggleButtonsTheme toggleButtonsTheme,
    Color buttonColor,
    Color secondaryHeaderColor,
    Color textSelectionColor,
    Color cursorColor,
    Color textSelectionHandleColor,
    Color backgroundColor,
    Color dialogBackgroundColor,
    Color indicatorColor,
    Color hintColor,
    Color errorColor,
    Color toggleableActiveColor,
    TextTheme textTheme,
    TextTheme primaryTextTheme,
    TextTheme accentTextTheme,
    InputDecorationTheme inputDecorationTheme,
    IconThemeData iconTheme,
    IconThemeData primaryIconTheme,
    IconThemeData accentIconTheme,
    SliderThemeData sliderTheme,
    TabBarTheme tabBarTheme,
    TooltipThemeData tooltipTheme,
    CardTheme cardTheme,
    ChipThemeData chipTheme,
    TargetPlatform platform,
    MaterialTapTargetSize materialTapTargetSize,
    bool applyElevationOverlayColor,
    PageTransitionsTheme pageTransitionsTheme,
    AppBarTheme appBarTheme,
    BottomAppBarTheme bottomAppBarTheme,
    ColorScheme colorScheme,
    DialogTheme dialogTheme,
    FloatingActionButtonThemeData floatingActionButtonTheme,
    Typography typography,
    CupertinoThemeData cupertinoOverrideTheme,
    SnackBarThemeData snackBarTheme,
    BottomSheetThemeData bottomSheetTheme,
    PopupMenuThemeData popupMenuTheme,
    MaterialBannerThemeData bannerTheme,
    DividerThemeData dividerTheme,
  }) {
    cupertinoOverrideTheme = cupertinoOverrideTheme?.noDefault();

    return CustomTheme<T>(
      themeData: ThemeData.raw(
        brightness: brightness ?? this.brightness,
        primaryColor: primaryColor ?? this.primaryColor,
        primaryColorBrightness:
            primaryColorBrightness ?? this.primaryColorBrightness,
        primaryColorLight: primaryColorLight ?? this.primaryColorLight,
        primaryColorDark: primaryColorDark ?? this.primaryColorDark,
        accentColor: accentColor ?? this.accentColor,
        accentColorBrightness:
            accentColorBrightness ?? this.accentColorBrightness,
        canvasColor: canvasColor ?? this.canvasColor,
        scaffoldBackgroundColor:
            scaffoldBackgroundColor ?? this.scaffoldBackgroundColor,
        bottomAppBarColor: bottomAppBarColor ?? this.bottomAppBarColor,
        cardColor: cardColor ?? this.cardColor,
        dividerColor: dividerColor ?? this.dividerColor,
        focusColor: focusColor ?? this.focusColor,
        hoverColor: hoverColor ?? this.hoverColor,
        highlightColor: highlightColor ?? this.highlightColor,
        splashColor: splashColor ?? this.splashColor,
        splashFactory: splashFactory ?? this.splashFactory,
        selectedRowColor: selectedRowColor ?? this.selectedRowColor,
        unselectedWidgetColor:
            unselectedWidgetColor ?? this.unselectedWidgetColor,
        disabledColor: disabledColor ?? this.disabledColor,
        buttonColor: buttonColor ?? this.buttonColor,
        buttonTheme: buttonTheme ?? this.buttonTheme,
        toggleButtonsTheme: toggleButtonsTheme ?? this.toggleButtonsTheme,
        secondaryHeaderColor: secondaryHeaderColor ?? this.secondaryHeaderColor,
        textSelectionColor: textSelectionColor ?? this.textSelectionColor,
        cursorColor: cursorColor ?? this.cursorColor,
        textSelectionHandleColor:
            textSelectionHandleColor ?? this.textSelectionHandleColor,
        backgroundColor: backgroundColor ?? this.backgroundColor,
        dialogBackgroundColor:
            dialogBackgroundColor ?? this.dialogBackgroundColor,
        indicatorColor: indicatorColor ?? this.indicatorColor,
        hintColor: hintColor ?? this.hintColor,
        errorColor: errorColor ?? this.errorColor,
        toggleableActiveColor:
            toggleableActiveColor ?? this.toggleableActiveColor,
        textTheme: textTheme ?? this.textTheme,
        primaryTextTheme: primaryTextTheme ?? this.primaryTextTheme,
        accentTextTheme: accentTextTheme ?? this.accentTextTheme,
        inputDecorationTheme: inputDecorationTheme ?? this.inputDecorationTheme,
        iconTheme: iconTheme ?? this.iconTheme,
        primaryIconTheme: primaryIconTheme ?? this.primaryIconTheme,
        accentIconTheme: accentIconTheme ?? this.accentIconTheme,
        sliderTheme: sliderTheme ?? this.sliderTheme,
        tabBarTheme: tabBarTheme ?? this.tabBarTheme,
        tooltipTheme: tooltipTheme ?? this.tooltipTheme,
        cardTheme: cardTheme ?? this.cardTheme,
        chipTheme: chipTheme ?? this.chipTheme,
        platform: platform ?? this.platform,
        materialTapTargetSize:
            materialTapTargetSize ?? this.materialTapTargetSize,
        applyElevationOverlayColor:
            applyElevationOverlayColor ?? this.applyElevationOverlayColor,
        pageTransitionsTheme: pageTransitionsTheme ?? this.pageTransitionsTheme,
        appBarTheme: appBarTheme ?? this.appBarTheme,
        bottomAppBarTheme: bottomAppBarTheme ?? this.bottomAppBarTheme,
        colorScheme: colorScheme ?? this.colorScheme,
        dialogTheme: dialogTheme ?? this.dialogTheme,
        floatingActionButtonTheme:
            floatingActionButtonTheme ?? this.floatingActionButtonTheme,
        typography: typography ?? this.typography,
        cupertinoOverrideTheme:
            cupertinoOverrideTheme ?? this.cupertinoOverrideTheme,
        snackBarTheme: snackBarTheme ?? this.snackBarTheme,
        bottomSheetTheme: bottomSheetTheme ?? this.bottomSheetTheme,
        popupMenuTheme: popupMenuTheme ?? this.popupMenuTheme,
        bannerTheme: bannerTheme ?? this.bannerTheme,
        dividerTheme: dividerTheme ?? this.dividerTheme,
      ),
      userData: this._userData,
    );
  }
}
